/* Things that can traditionally be controlled by the user theme, but that we *must*
 * have regardless, for a usable application.  A couple things that we cannot do without. */

/* Inactive F3 pane shading */

.nemo-window .nemo-inactive-pane .view, .nemo-window .nemo-inactive-pane iconview {
    background-color: @theme_unfocused_bg_color;
}

/* Rename box styling in the icon view. */

.nemo-window .nemo-window-pane widget.entry {
    border: 1px solid;
    border-radius: 3px;
    color: @theme_fg_color;
    border-color: @theme_selected_bg_color;
    background-color: @theme_bg_color;
}

.nemo-window .nemo-window-pane widget.entry:selected {
    border: 1px solid;
    border-radius: 3px;
    color: @theme_selected_fg_color;
    border-color: @theme_selected_bg_color;
    background-color: @theme_selected_bg_color;
}
