#ifndef NEMO_ICON_NAMES_H
#define NEMO_ICON_NAMES_H

/* Icons for places */
#define NEMO_ICON_COMPUTER		"computer"
#define NEMO_ICON_DESKTOP		"user-desktop"
#define NEMO_ICON_FILESYSTEM	"drive-harddisk"
#define NEMO_ICON_FOLDER		"folder"
#define NEMO_ICON_FOLDER_REMOTE	"folder-remote"
#define NEMO_ICON_HOME		"user-home"
#define NEMO_ICON_NETWORK		"network-workgroup"
#define NEMO_ICON_NETWORK_SERVER	"network-server"
#define NEMO_ICON_SEARCH		"system-search"
#define NEMO_ICON_TRASH		"user-trash"
#define NEMO_ICON_TRASH_FULL	"user-trash-full"
#define NEMO_ICON_DELETE		"edit-delete-symbolic"
#define NEMO_ICON_FAVORITES "xapp-user-favorites"

#define NEMO_ICON_FOLDER_DESKTOP    "user-desktop"
#define NEMO_ICON_FOLDER_DOCUMENTS  "folder-documents"
#define NEMO_ICON_FOLDER_DOWNLOAD   "folder-download"
#define NEMO_ICON_FOLDER_MUSIC      "folder-music"
#define NEMO_ICON_FOLDER_PICTURES   "folder-pictures"
#define NEMO_ICON_FOLDER_PUBLIC_SHARE "folder-publicshare"
#define NEMO_ICON_FOLDER_TEMPLATES  "folder-templates"
#define NEMO_ICON_FOLDER_VIDEOS     "folder-videos"
#define NEMO_ICON_FOLDER_SAVED_SEARCH "folder-saved-search"

/* Other icons */
#define NEMO_ICON_TEMPLATE		"text-x-generic-template"

/* Icons not provided by fd.o naming spec or nemo itself */
#define NEMO_ICON_BURN		"nemo-cd-burner"

#define NEMO_ICON_SYMBOLIC_COMPUTER      "computer-symbolic"
#define NEMO_ICON_SYMBOLIC_DESKTOP       "user-desktop-symbolic"
#define NEMO_ICON_SYMBOLIC_FILESYSTEM    "drive-harddisk-symbolic"
#define NEMO_ICON_SYMBOLIC_FOLDER        "folder-symbolic"
#define NEMO_ICON_SYMBOLIC_FOLDER_REMOTE "folder-remote-symbolic"
#define NEMO_ICON_SYMBOLIC_HOME      "user-home-symbolic"
#define NEMO_ICON_SYMBOLIC_NETWORK       "network-workgroup-symbolic"
#define NEMO_ICON_SYMBOLIC_NETWORK_SERVER    "network-server-symbolic"
#define NEMO_ICON_SYMBOLIC_TRASH     "user-trash-symbolic"
#define NEMO_ICON_SYMBOLIC_TRASH_FULL    "user-trash-full-symbolic"

#define NEMO_ICON_SYMBOLIC_FOLDER_DOCUMENTS  "folder-documents-symbolic"
#define NEMO_ICON_SYMBOLIC_FOLDER_DOWNLOAD   "folder-download-symbolic"
#define NEMO_ICON_SYMBOLIC_FOLDER_MUSIC      "folder-music-symbolic"
#define NEMO_ICON_SYMBOLIC_FOLDER_PICTURES   "folder-pictures-symbolic"
#define NEMO_ICON_SYMBOLIC_FOLDER_PUBLIC_SHARE "folder-publicshare-symbolic"
#define NEMO_ICON_SYMBOLIC_FOLDER_TEMPLATES  "folder-templates-symbolic"
#define NEMO_ICON_SYMBOLIC_FOLDER_VIDEOS     "folder-videos-symbolic"
#define NEMO_ICON_SYMBOLIC_FOLDER_SAVED_SEARCH "folder-saved-search-symbolic"
#define NEMO_ICON_SYMBOLIC_FOLDER_RECENT "document-open-recent-symbolic"
#define NEMO_ICON_SYMBOLIC_FOLDER_FAVORITES "xapp-user-favorites-symbolic"

#define NEMO_ICON_SYMBOLIC_MISSING_BOOKMARK "nemo-bookmark-not-found-symbolic"








#endif /* NEMO_ICON_NAMES_H */

