/* For Places Sidebar diskfull indicators */

.places-treeview {
    -NemoPlacesTreeView-disk-full-bg-color: shade(@theme_bg_color, .65);
    -NemoPlacesTreeView-disk-full-fg-color: shade(@theme_selected_bg_color, 1.0);
    -NemoPlacesTreeView-disk-full-bar-width: 2px;
    -NemoPlacesTreeView-disk-full-bar-radius: 1px;
    -NemoPlacesTreeView-disk-full-bottom-padding: 1px;
    -NemoPlacesTreeView-disk-full-max-length: 70px;
}

.places-treeview:selected {
    -NemoPlacesTreeView-disk-full-bg-color: shade(@theme_bg_color, 2.0);
    -NemoPlacesTreeView-disk-full-fg-color: shade(@theme_fg_color, 2.0);
}

.places-treeview:hover {
}

.places-treeview:selected:hover {
}
